{-|
[@ISO639-1@]        fi

[@ISO639-2@]        fin

[@ISO639-3@]        fin

[@Native name@]     suomi

[@English name@]    Finnish
-}

module Text.Numeral.Language.FIN.TestData
    ( cardinals
    , ordinals
    , multiplicatives
    , multipliers
    ) where


--------------------------------------------------------------------------------
-- Imports
--------------------------------------------------------------------------------

import "base" Data.Function ( ($) )
import "base" Prelude     ( Integral, (+) )
import "numerals" Text.Numeral.Grammar
import "numerals" Text.Numeral.Grammar ( defaultInflection )
import "numerals" Text.Numeral.Misc ( dec )
import "this" Text.Numeral.Test ( TestData )


--------------------------------------------------------------------------------
-- Test data
--------------------------------------------------------------------------------

-- Sources:
--   http://www.marijn.org/everything-is-4/counting-0-to-100/finnish
--   http://www.languagesandnumbers.com/how-to-count-in-finnish/en/fin/
--   http://en.wikipedia.org/wiki/Finnish_numerals
--   http://mylanguages.org/finnish_numbers.php
--   http://users.jyu.fi/~pamakine/kieli/suomi/numeraalit/numeroten.html

cardinals :: (Integral i) => TestData i
cardinals =
  [ ( "singular nominative"
    , singular $ nominative defaultInflection
    , [ (0, "nolla")
      , (1, "yksi")
      , (2, "kaksi")
      , (3, "kolme")
      , (4, "neljä")
      , (5, "viisi")
      , (6, "kuusi")
      , (7, "seitsemän")
      , (8, "kahdeksan")
      , (9, "yhdeksän")
      , (10, "kymmenen")
      , (11, "yksitoista")
      , (12, "kaksitoista")
      , (13, "kolmetoista")
      , (14, "neljätoista")
      , (15, "viisitoista")
      , (16, "kuusitoista")
      , (17, "seitsemäntoista")
      , (18, "kahdeksantoista")
      , (19, "yhdeksäntoista")
      , (20, "kaksikymmentä")
      , (21, "kaksikymmentäyksi")
      , (22, "kaksikymmentäkaksi")
      , (23, "kaksikymmentäkolme")
      , (24, "kaksikymmentäneljä")
      , (25, "kaksikymmentäviisi")
      , (26, "kaksikymmentäkuusi")
      , (27, "kaksikymmentäseitsemän")
      , (28, "kaksikymmentäkahdeksan")
      , (29, "kaksikymmentäyhdeksän")
      , (30, "kolmekymmentä")
      , (31, "kolmekymmentäyksi")
      , (32, "kolmekymmentäkaksi")
      , (33, "kolmekymmentäkolme")
      , (34, "kolmekymmentäneljä")
      , (35, "kolmekymmentäviisi")
      , (36, "kolmekymmentäkuusi")
      , (37, "kolmekymmentäseitsemän")
      , (38, "kolmekymmentäkahdeksan")
      , (39, "kolmekymmentäyhdeksän")
      , (40, "neljäkymmentä")
      , (41, "neljäkymmentäyksi")
      , (42, "neljäkymmentäkaksi")
      , (43, "neljäkymmentäkolme")
      , (44, "neljäkymmentäneljä")
      , (45, "neljäkymmentäviisi")
      , (46, "neljäkymmentäkuusi")
      , (47, "neljäkymmentäseitsemän")
      , (48, "neljäkymmentäkahdeksan")
      , (49, "neljäkymmentäyhdeksän")
      , (50, "viisikymmentä")
      , (51, "viisikymmentäyksi")
      , (52, "viisikymmentäkaksi")
      , (53, "viisikymmentäkolme")
      , (54, "viisikymmentäneljä")
      , (55, "viisikymmentäviisi")
      , (56, "viisikymmentäkuusi")
      , (57, "viisikymmentäseitsemän")
      , (58, "viisikymmentäkahdeksan")
      , (59, "viisikymmentäyhdeksän")
      , (60, "kuusikymmentä")
      , (61, "kuusikymmentäyksi")
      , (62, "kuusikymmentäkaksi")
      , (63, "kuusikymmentäkolme")
      , (64, "kuusikymmentäneljä")
      , (65, "kuusikymmentäviisi")
      , (66, "kuusikymmentäkuusi")
      , (67, "kuusikymmentäseitsemän")
      , (68, "kuusikymmentäkahdeksan")
      , (69, "kuusikymmentäyhdeksän")
      , (70, "seitsemänkymmentä")
      , (71, "seitsemänkymmentäyksi")
      , (72, "seitsemänkymmentäkaksi")
      , (73, "seitsemänkymmentäkolme")
      , (74, "seitsemänkymmentäneljä")
      , (75, "seitsemänkymmentäviisi")
      , (76, "seitsemänkymmentäkuusi")
      , (77, "seitsemänkymmentäseitsemän")
      , (78, "seitsemänkymmentäkahdeksan")
      , (79, "seitsemänkymmentäyhdeksän")
      , (80, "kahdeksankymmentä")
      , (81, "kahdeksankymmentäyksi")
      , (82, "kahdeksankymmentäkaksi")
      , (83, "kahdeksankymmentäkolme")
      , (84, "kahdeksankymmentäneljä")
      , (85, "kahdeksankymmentäviisi")
      , (86, "kahdeksankymmentäkuusi")
      , (87, "kahdeksankymmentäseitsemän")
      , (88, "kahdeksankymmentäkahdeksan")
      , (89, "kahdeksankymmentäyhdeksän")
      , (90, "yhdeksänkymmentä")
      , (91, "yhdeksänkymmentäyksi")
      , (92, "yhdeksänkymmentäkaksi")
      , (93, "yhdeksänkymmentäkolme")
      , (94, "yhdeksänkymmentäneljä")
      , (95, "yhdeksänkymmentäviisi")
      , (96, "yhdeksänkymmentäkuusi")
      , (97, "yhdeksänkymmentäseitsemän")
      , (98, "yhdeksänkymmentäkahdeksan")
      , (99, "yhdeksänkymmentäyhdeksän")
      , (100, "sata")
      , (101, "satayksi")
      , (123, "satakaksikymmentäkolme")
      , (200, "kaksisataa")
      , (300, "kolmesataa")
      , (357, "kolmesataaviisikymmentäseitsemän")
      , (400, "neljäsataa")
      , (500, "viisisataa")
      , (567, "viisisataakuusikymmentäseitsemän")
      , (600, "kuusisataa")
      , (700, "seitsemänsataa")
      , (800, "kahdeksansataa")
      , (900, "yhdeksänsataa")
      , (999, "yhdeksänsataayhdeksänkymmentäyhdeksän")
      , (1000, "tuhat")
      , (1001, "tuhatyksi")
      , (2000, "kaksituhatta")
      , (3000, "kolmetuhatta")
      , (3456, "kolmetuhattaneljäsataaviisikymmentäkuusi")
      , (3721, "kolmetuhattaseitsemänsataakaksikymmentäyksi")
      , (4000, "neljätuhatta")
      , (5000, "viisituhatta")
      , (6000, "kuusituhatta")
      , (7000, "seitsemäntuhatta")
      , (8000, "kahdeksantuhatta")
      , (9000, "yhdeksäntuhatta")
      , (10000, "kymmenentuhatta")
      , (11000, "toistatuhatta")
      , (12000, "toistakaksituhatta")
      , (13000, "toistakolmetuhatta")
      , (dec 5, "satatuhatta")
      , (2 * dec 5, "kaksisataatuhatta")
      , (dec 6, "miljoona")
      , (dec 6 + 1, "miljoonayksi")
      , (2 * dec 6, "kaksimiljoonaa")
      , (32534756, "kolmekymmentäkaksimiljoonaaviisisataakolmekymmentäneljätuhattaseitsemänsataaviisikymmentäkuusi")
      , (dec 9, "miljardi")
      , (2 * dec 9, "kaksimiljardia")
      , (3 * dec 9, "kolmemiljardia")
      , (dec 12, "biljoona")
      , (3 * dec 12, "kolmebiljoonaa")
      ]
    )
  , ( "singular accusative"
    , singular $ accusative defaultInflection
    , [ (0, "nolla")
      , (1, "yksi")
      , (2, "kaksi")
      , (3, "kolme")
      , (4, "neljä")
      , (5, "viisi")
      , (6, "kuusi")
      , (7, "seitsemän")
      , (8, "kahdeksan")
      , (9, "yhdeksän")
      , (10, "kymmenen")
      , (11, "yksitoista")
      , (12, "kaksitoista")
      , (13, "kolmetoista")
      , (14, "neljätoista")
      , (15, "viisitoista")
      , (16, "kuusitoista")
      , (17, "seitsemäntoista")
      , (18, "kahdeksantoista")
      , (19, "yhdeksäntoista")
      , (20, "kaksikymmentä")
      , (21, "kaksikymmentäyksi")
      , (100, "sata")
      , (1000, "tuhat")
      ]
    )
  , ( "singular genitive"
    , singular $ genitive defaultInflection
    , [ (0, "nollan")
      , (1, "yhden")
      , (2, "kahden")
      , (3, "kolmen")
      , (4, "neljän")
      , (5, "viiden")
      , (6, "kuuden")
      , (7, "seitsemän")
      , (8, "kahdeksan")
      , (9, "yhdeksän")
      , (10, "kymmenen")
      , (11, "yhdentoista")
      , (12, "kahdentoista")
      , (13, "kolmentoista")
      , (14, "neljäntoista")
      , (15, "viidentoista")
      , (16, "kuudentoista")
      , (17, "seitsemäntoista")
      , (18, "kahdeksantoista")
      , (19, "yhdeksäntoista")
      , (20, "kahdenkymmenen")
      , (21, "kahdenkymmenenyhden")
      , (100, "sadan")
      , (357, "kolmensadanviidenkymmenenseitsemän")
      , (1000, "tuhannen")
      ]
    )
  , ( "singular partitive"
    , singular $ partitive defaultInflection
    , [ (0, "nollaa")
      , (1, "yhtä")
      , (2, "kahta")
      , (3, "kolmea")
      , (4, "neljää")
      , (5, "viittä")
      , (6, "kuutta")
      , (7, "seitsemää")
      , (8, "kahdeksaa")
      , (9, "yhdeksää")
      , (10, "kymmentä")
      , (11, "yhtätoista")
      , (12, "kahtatoista")
      , (13, "kolmeatoista")
      , (14, "neljäätoista")
      , (15, "viittätoista")
      , (16, "kuuttatoista")
      , (17, "seitsemäätoista")
      , (18, "kahdeksaatoista")
      , (19, "yhdeksäätoista")
      , (20, "kahtakymmentä")
      , (21, "kahtakymmentäyhtä")
      , (100, "sataa")
      , (357, "kolmeasataaviittäkymmentäseitsemää")
      , (1000, "tuhatta")
      ]
    )
  , ( "singular essive"
    , singular $ essive defaultInflection
    , [ (0, "nollana")
      , (1, "yhtenä")
      , (2, "kahtena")
      , (3, "kolmena")
      , (4, "neljänä")
      , (5, "viitenä")
      , (6, "kuutena")
      , (7, "seitsemänä")
      , (8, "kahdeksana")
      , (9, "yhdeksänä")
      , (10, "kymmenenä")
      , (11, "yhtenätoista")
      , (12, "kahtenatoista")
      , (13, "kolmenatoista")
      , (14, "neljänätoista")
      , (15, "viitenätoista")
      , (16, "kuutenatoista")
      , (17, "seitsemänätoista")
      , (18, "kahdeksanatoista")
      , (19, "yhdeksänätoista")
      , (20, "kahtenakymmenenä")
      , (21, "kahdessakymmenessäyhdessä")
      , (100, "satana")
      , (357, "kolmenasatanaviitenäkymmenenäseitsemänä")
      , (1000, "tuhantena")
      ]
    )
  , ( "singular translative"
    , singular $ translative defaultInflection
    , [ (0, "nollaksi")
      , (1, "yhdeksi")
      , (2, "kahdeksi")
      , (3, "kolmeksi")
      , (4, "neljäksi")
      , (5, "viideksi")
      , (6, "kuudeksi")
      , (7, "seitsemäksi")
      , (8, "kahdeksaksi")
      , (9, "yhdeksäksi")
      , (10, "kymmeneksi")
      , (11, "yhdeksitoista")
      , (12, "kahdeksitoista")
      , (13, "kolmeksitoista")
      , (14, "neljäksitoista")
      , (15, "viideksitoista")
      , (16, "kuudeksitoista")
      , (17, "seitsemäksitoista")
      , (18, "kahdeksaksitoista")
      , (19, "yhdeksäksitoista")
      , (20, "kahdeksikymmeneksi")
      , (21, "kahdeksikymmeneksiyhdeksi")
      , (100, "sadaksi")
      , (1000, "tuhanneksi")
      ]
    )
  , ( "singular locative inessive"
    , singular $ locativeInessive defaultInflection
    , [ (0, "nollassa")
      , (1, "yhdessä")
      , (2, "kahdessa")
      , (3, "kolmessa")
      , (4, "neljässä")
      , (5, "viidessä")
      , (6, "kuudessa")
      , (7, "seitsemässä")
      , (8, "kahdeksassa")
      , (9, "yhdeksässä")
      , (10, "kymmenessä")
      , (11, "yhdessätoista")
      , (12, "kahdessatoista")
      , (13, "kolmessatoista")
      , (14, "neljässätoista")
      , (15, "viidessätoista")
      , (16, "kuudessatoista")
      , (17, "seitsemässätoista")
      , (18, "kahdeksassatoista")
      , (19, "yhdeksässätoista")
      , (20, "kahdessakymmenessä")
      , (21, "kahdessakymmenessäyhdessä")
      , (100, "sadassa")
      , (1000, "tuhannessa")
      ]
    )
  , ( "singular locative elative"
    , singular $ locativeElative defaultInflection
    , [ (0, "nollasta")
      , (1, "yhdestä")
      , (2, "kahdesta")
      , (3, "kolmesta")
      , (4, "neljästä")
      , (5, "viidestä")
      , (6, "kuudesta")
      , (7, "seitsemästä")
      , (8, "kahdeksasta")
      , (9, "yhdeksästä")
      , (10, "kymmenestä")
      , (11, "yhdestätoista")
      , (12, "kahdestatoista")
      , (13, "kolmestatoista")
      , (14, "neljästätoista")
      , (15, "viidestätoista")
      , (16, "kuudestatoista")
      , (17, "seitsemästätoista")
      , (18, "kahdeksastatoista")
      , (19, "yhdeksästätoista")
      , (20, "kahdestakymmenestä")
      , (21, "kahdestakymmenestäyhdestä")
      , (100, "sadasta")
      , (1000, "tuhannesta")
      ]
    )
  , ( "singular locative illative"
    ,  singular $ locativeIllative defaultInflection
    , [ (0, "nollaan")
      , (1, "yhteen")
      , (2, "kahteen")
      , (3, "kolmeen")
      , (4, "neljään")
      , (5, "viiteen")
      , (6, "kuuteen")
      , (7, "seitsemään")
      , (8, "kahdeksaan")
      , (9, "yhdeksään")
      , (10, "kymmeneen")
      , (11, "yhteentoista")
      , (12, "kahteentoista")
      , (13, "kolmeentoista")
      , (14, "neljääntoista")
      , (15, "viiteentoista")
      , (16, "kuuteentoista")
      , (17, "seitsemääntoista")
      , (18, "kahdeksaantoista")
      , (19, "yhdeksääntoista")
      , (20, "kahteenkymmeneen")
      , (21, "kahteenkymmeneenyhteen")
      , (100, "sataan")
      , (1000, "tuhanteen")
      ]
    )
  , ( "singular locative adessive"
    , singular $ locativeAdessive defaultInflection
    , [ (0, "nollalla")
      , (1, "yhdellä")
      , (2, "kahdella")
      , (3, "kolmella")
      , (4, "neljällä")
      , (5, "viidellä")
      , (6, "kuudella")
      , (7, "seitsemällä")
      , (8, "kahdeksalla")
      , (9, "yhdeksällä")
      , (10, "kymmenellä")
      , (11, "yhdellätoista")
      , (12, "kahdellatoista")
      , (13, "kolmellatoista")
      , (14, "neljällätoista")
      , (15, "viidellätoista")
      , (16, "kuudellatoista")
      , (17, "seitsemällätoista")
      , (18, "kahdeksallatoista")
      , (19, "yhdeksällätoista")
      , (20, "kahdellakymmenellä")
      , (21, "kahdellakymmenelläyhdellä")
      , (100, "sadalla")
      , (1000, "tuhannella")
      ]
    )
  , ( "singular locative ablative"
    , singular $ locativeAblative defaultInflection
    , [ (0, "nollalta")
      , (1, "yhdeltä")
      , (2, "kahdelta")
      , (3, "kolmelta")
      , (4, "neljältä")
      , (5, "viideltä")
      , (6, "kuudelta")
      , (7, "seitsemältä")
      , (8, "kahdeksalta")
      , (9, "yhdeksältä")
      , (10, "kymmeneltä")
      , (11, "yhdeltätoista")
      , (12, "kahdeltatoista")
      , (13, "kolmeltatoista")
      , (14, "neljältätoista")
      , (15, "viideltätoista")
      , (16, "kuudeltatoista")
      , (17, "seitsemältätoista")
      , (18, "kahdeksaltatoista")
      , (19, "yhdeksältätoista")
      , (20, "kahdeltakymmeneltä")
      , (21, "kahdeltakymmeneltäyhdeltä")
      , (100, "sadalta")
      , (1000, "tuhannelta")
      ]
    )
  , ( "singular locative allative"
    , singular $ locativeAllative defaultInflection
    , [ (0, "nollalle")
      , (1, "yhdelle")
      , (2, "kahdelle")
      , (3, "kolmelle")
      , (4, "neljälle")
      , (5, "viidelle")
      , (6, "kuudelle")
      , (7, "seitsemälle")
      , (8, "kahdeksalle")
      , (9, "yhdeksälle")
      , (10, "kymmenelle")
      , (11, "yhdelletoista")
      , (12, "kahdelletoista")
      , (13, "kolmelletoista")
      , (14, "neljälletoista")
      , (15, "viidelletoista")
      , (16, "kuudelletoista")
      , (17, "seitsemälletoista")
      , (18, "kahdeksalletoista")
      , (19, "yhdeksälletoista")
      , (20, "kahdellekymmenelle")
      , (21, "kahdellekymmenelleyhdelle")
      , (100, "sadalle")
      , (1000, "tuhannelle")
      ]
    )
  , ( "singular abessive"
    , singular $ abessive defaultInflection
    , [ (0, "nollatta")
      , (1, "yhdettä")
      , (2, "kahdetta")
      , (3, "kolmetta")
      , (4, "neljättä")
      , (5, "viidettä")
      , (6, "kuudetta")
      , (7, "seitsemättä")
      , (8, "kahdeksatta")
      , (9, "yhdeksättä")
      , (10, "kymmenettä")
      , (11, "yhdettätoista")
      , (12, "kahdettatoista")
      , (13, "kolmettatoista")
      , (14, "neljättätoista")
      , (15, "viidettätoista")
      , (16, "kuudettatoista")
      , (17, "seitsemättätoista")
      , (18, "kahdeksattatoista")
      , (19, "yhdeksättätoista")
      , (20, "kahdettakymmenettä")
      , (21, "kahdettakymmenettäyhdettä")
      , (100, "sadatta")
      , (1000, "tuhannetta")
      ]
    )
  , ( "singular super essive"
    , singular $ superEssive defaultInflection
    , [ (1, "yhtäällä")
      , (2, "kahtaalla")
      ]
    )
  , ( "singular delative"
    , singular $ delative defaultInflection
    , [ (1, "yhtäältä")
      , (2, "kahtaalta")
      ]
    )
  , ( "singular sublative"
    , singular $ sublative defaultInflection
    , [ (1, "yhtäälle")
      , (2, "kahtaalle")
      ]
    )
  , ( "singular lative"
    , singular $ lative defaultInflection
    , [ (1, "yhä")
      , (2, "kahtia")
      , (3, "kolmia")
      ]
    )
  , ( "singular multiplicative"
    , singular $ multiplicative defaultInflection
    , [ (1, "yhdesti")
      , (2, "kahdesti")
      , (3, "kolmesti")
      ]
    )
  , ( "plural nominative"
    , plural $ nominative defaultInflection
    , [ (0, "nollat")
      , (1, "yhdet")
      , (2, "kahdet")
      , (3, "kolmet")
      , (4, "neljät")
      , (5, "viidet")
      , (6, "kuudet")
      , (7, "seitsemät")
      , (8, "kahdeksat")
      , (9, "yhdeksät")
      , (10, "kymmenet")
      , (11, "yhdettoista")
      , (12, "kahdettoista")
      , (13, "kolmettoista")
      , (14, "neljättoista")
      , (15, "viidettoista")
      , (16, "kuudettoista")
      , (17, "seitsemättoista")
      , (18, "kahdeksattoista")
      , (19, "yhdeksättoista")
      , (20, "kahdetkymmenet")
      , (21, "kahdetkymmenetyhdet")
      , (100, "sadat")
      , (1000, "tuhannet")
      ]
    )
  , ( "plural accusative"
    , plural $ accusative defaultInflection
    , [ (0, "nollat")
      , (1, "yhdet")
      , (2, "kahdet")
      , (3, "kolmet")
      , (4, "neljät")
      , (5, "viidet")
      , (6, "kuudet")
      , (7, "seitsemät")
      , (8, "kahdeksat")
      , (9, "yhdeksät")
      , (10, "kymmenet")
      , (11, "yhdettoista")
      , (12, "kahdettoista")
      , (13, "kolmettoista")
      , (14, "neljättoista")
      , (15, "viidettoista")
      , (16, "kuudettoista")
      , (17, "seitsemättoista")
      , (18, "kahdeksattoista")
      , (19, "yhdeksättoista")
      , (20, "kahdetkymmenet")
      , (21, "kahdetkymmenetyhdet")
      , (100, "sadat")
      , (1000, "tuhannet")
      ]
    )
  , ( "plural genitive"
    , plural $ genitive defaultInflection
    , [ (0, "nollien")
      , (1, "yksien")
      , (2, "kaksien")
      , (3, "kolmien")
      , (4, "neljien")
      , (5, "viisien")
      , (6, "kuusien")
      , (7, "seitsemien")
      , (8, "kahdeksien")
      , (9, "yhdeksien")
      , (10, "kymmenien")
      , (11, "yksientoista")
      , (12, "kaksientoista")
      , (13, "kolmientoista")
      , (14, "neljientoista")
      , (15, "viisientoista")
      , (16, "kuusientoista")
      , (17, "seitsemientoista")
      , (18, "kahdeksientoista")
      , (19, "yhdeksientoista")
      , (20, "kaksienkymmenien")
      , (21, "kaksienkymmenienyksien")
      , (100, "satojen")
      , (1000, "tuhansien")
      ]
    )
  , ( "plural partitive"
    , plural $ partitive defaultInflection
    , [ (0, "nollia")
      , (1, "yksiä")
      , (2, "kaksia")
      , (3, "kolmia")
      , (4, "neljiä")
      , (5, "viisiä")
      , (6, "kuusia")
      , (7, "seitsemiä")
      , (8, "kahdeksia")
      , (9, "yhdeksiä")
      , (10, "kymmeniä")
      , (11, "yksiätoista")
      , (12, "kaksiatoista")
      , (13, "kolmiatoista")
      , (14, "neljiätoista")
      , (15, "viittätoista")
      , (16, "kuuttatoista")
      , (17, "seitsemiätoista")
      , (18, "kahdeksiatoista")
      , (19, "yhdeksiätoista")
      , (20, "kaksiakymmeniä")
      , (21, "kaksiakymmeniäyksiä")
      , (100, "satoja")
      , (1000, "tuhansia")
      ]
    )
  , ( "plural essive"
    , plural $ essive defaultInflection
    , [ (0, "nollina")
      , (1, "yksinä")
      , (2, "kaksina")
      , (3, "kolmina")
      , (4, "neljinä")
      , (5, "viisinä")
      , (6, "kuusina")
      , (7, "seitseminä")
      , (8, "kahdeksina")
      , (9, "yhdeksinä")
      , (10, "kymmeninä")
      , (11, "yksinätoista")
      , (12, "kaksinatoista")
      , (13, "kolminatoista")
      , (14, "neljinätoista")
      , (15, "viitenätoista")
      , (16, "kuutenatoista")
      , (17, "seitseminätoista")
      , (18, "kahdeksinatoista")
      , (19, "yhdeksinätoista")
      , (20, "kaksinakymmeninä")
      , (21, "kaksinakymmeninäyksinä")
      , (100, "satoina")
      , (1000, "tuhansina")
      ]
    )
  , ( "plural translative"
    , plural $ translative defaultInflection
    , [ (0, "nolliksi")
      , (1, "yksiksi")
      , (2, "kaksiksi")
      , (3, "kolmiksi")
      , (4, "neljiksi")
      , (5, "viisiksi")
      , (6, "kuusiksi")
      , (7, "seitsemiksi")
      , (8, "kahdeksiksi")
      , (9, "yhdeksiksi")
      , (10, "kymmeniksi")
      , (11, "yksiksitoista")
      , (12, "kaksiksitoista")
      , (13, "kolmiksitoista")
      , (14, "neljiksitoista")
      , (15, "viideksitoista")
      , (16, "kuudeksitoista")
      , (17, "seitsemiksitoista")
      , (18, "kahdeksiksitoista")
      , (19, "yhdeksiksitoista")
      , (20, "kaksiksikymmeniksi")
      , (21, "kaksiksikymmeniksiyksiksi")
      , (100, "sadoiksi")
      , (1000, "tuhansiksi")
      ]
    )
  , ( "plural locative inessive"
    , plural $ locativeInessive defaultInflection
    , [ (0, "nollissa")
      , (1, "yksissä")
      , (2, "kaksissa")
      , (3, "kolmissa")
      , (4, "neljissä")
      , (5, "viisissä")
      , (6, "kuusissa")
      , (7, "seitsemissä")
      , (8, "kahdeksissa")
      , (9, "yhdeksissä")
      , (10, "kymmenissä")
      , (11, "yksissätoista")
      , (12, "kaksissatoista")
      , (13, "kolmissatoista")
      , (14, "neljissätoista")
      , (15, "viidessätoista")
      , (16, "kuudessatoista")
      , (17, "seitsemissätoista")
      , (18, "kahdeksissatoista")
      , (19, "yhdeksissätoista")
      , (20, "kaksissakymmenissä")
      , (21, "kaksissakymmenissäyksissä")
      , (100, "sadoissa")
      , (1000, "tuhansissa")
      ]
    )
  , ( "plural locative elative"
    , plural $ locativeElative defaultInflection
    , [ (0, "nollista")
      , (1, "yksistä")
      , (2, "kaksista")
      , (3, "kolmista")
      , (4, "neljistä")
      , (5, "viisistä")
      , (6, "kuusista")
      , (7, "seitsemistä")
      , (8, "kahdeksista")
      , (9, "yhdeksistä")
      , (10, "kymmenistä")
      , (11, "yksistätoista")
      , (12, "kaksistatoista")
      , (13, "kolmistatoista")
      , (14, "neljistätoista")
      , (15, "viidestätoista")
      , (16, "kuudestatoista")
      , (17, "seitsemistätoista")
      , (18, "kahdeksistatoista")
      , (19, "yhdeksistätoista")
      , (20, "kaksistakymmenistä")
      , (21, "kaksistakymmenistäyksistä")
      , (100, "sadoista")
      , (1000, "tuhansista")
      ]
    )
  , ( "plural locative illative"
    , plural $ locativeIllative defaultInflection
    , [ (0, "nolliin")
      , (1, "yksiin")
      , (2, "kaksiin")
      , (3, "kolmiin")
      , (4, "neljiin")
      , (5, "viisiin")
      , (6, "kuusiin")
      , (7, "seitsemiin")
      , (8, "kahdeksiin")
      , (9, "yhdeksiin")
      , (10, "kymmeniin")
      , (11, "yksiintoista")
      , (12, "kaksiintoista")
      , (13, "kolmiintoista")
      , (14, "neljiintoista")
      , (15, "viiteentoista")
      , (16, "kuuteentoista")
      , (17, "seitsemiintoista")
      , (18, "kahdeksiintoista")
      , (19, "yhdeksiintoista")
      , (20, "kaksiinkymmeniin")
      , (21, "kaksiinkymmeniinyksiin")
      , (100, "satoihin")
      , (1000, "tuhansiin")
      ]
    )
  , ( "plural locative adessive"
    , plural $ locativeAdessive defaultInflection
    , [ (0, "nollilla")
      , (1, "yksillä")
      , (2, "kaksilla")
      , (3, "kolmilla")
      , (4, "neljillä")
      , (5, "viisillä")
      , (6, "kuusilla")
      , (7, "seitsemillä")
      , (8, "kahdeksilla")
      , (9, "yhdeksillä")
      , (10, "kymmenillä")
      , (11, "yksillätoista")
      , (12, "kaksillatoista")
      , (13, "kolmillatoista")
      , (14, "neljillätoista")
      , (15, "viidellätoista")
      , (16, "kuudellatoista")
      , (17, "seitsemillätoista")
      , (18, "kahdeksillatoista")
      , (19, "yhdeksillätoista")
      , (20, "kaksillakymmenillä")
      , (21, "kaksillakymmenilläyksillä")
      , (100, "sadoilla")
      , (1000, "tuhansilla")
      ]
    )
  , ( "plural locative ablative"
    , plural $ locativeAblative defaultInflection
    , [ (0, "nollilta")
      , (1, "yksiltä")
      , (2, "kaksilta")
      , (3, "kolmilta")
      , (4, "neljiltä")
      , (5, "viisiltä")
      , (6, "kuusilta")
      , (7, "seitsemiltä")
      , (8, "kahdeksilta")
      , (9, "yhdeksiltä")
      , (10, "kymmeniltä")
      , (11, "yksiltätoista")
      , (12, "kaksiltatoista")
      , (13, "kolmiltatoista")
      , (14, "neljiltätoista")
      , (15, "viideltätoista")
      , (16, "kuudeltatoista")
      , (17, "seitsemiltätoista")
      , (18, "kahdeksiltatoista")
      , (19, "yhdeksiltätoista")
      , (20, "kaksiltakymmeniltä")
      , (21, "kaksiltakymeniltäyksiltä")
      , (100, "sadoilta")
      , (1000, "tuhansilta")
      ]
    )
  , ( "plural locative allative"
    , plural $ locativeAllative defaultInflection
    , [ (0, "nollille")
      , (1, "yksille")
      , (2, "kaksille")
      , (3, "kolmille")
      , (4, "neljille")
      , (5, "viisille")
      , (6, "kuusille")
      , (7, "seitsemille")
      , (8, "kahdeksille")
      , (9, "yhdeksille")
      , (10, "kymmenille")
      , (11, "yksilletoista")
      , (12, "kaksilletoista")
      , (13, "kolmilletoista")
      , (14, "neljilletoista")
      , (15, "viidelletoista")
      , (16, "kuudelletoista")
      , (17, "seitsemilletoista")
      , (18, "kahdeksilletoista")
      , (19, "yhdeksilletoista")
      , (20, "kaksillekymmenille")
      , (21, "kaksillekymmenilleyksille")
      , (100, "sadoille")
      , (1000, "tuhansille")
      ]
    )
  , ( "plural abessive"
    , plural $ abessive defaultInflection
    , [ (0, "nollitta")
      , (1, "yksittä")
      , (2, "kaksitta")
      , (3, "kolmitta")
      , (4, "neljittä")
      , (5, "viisittä")
      , (6, "kuusitta")
      , (7, "seitsemittä")
      , (8, "kahdeksitta")
      , (9, "yhdeksittä")
      , (10, "kymmenittä")
      , (11, "yksittätoista")
      , (12, "kaksittatoista")
      , (13, "kolmittatoista")
      , (14, "neljittätoista")
      , (15, "viidettätoista")
      , (16, "kuudettatoista")
      , (17, "seitsemittätoista")
      , (18, "kahdeksittatoista")
      , (19, "yhdeksittätoista")
      , (20, "kaksittakymmenittä")
      , (21, "kaksittakymmenittäyksittä")
      , (100, "sadoitta")
      , (1000, "tuhansitta")
      ]
    )
  , ( "plural comitative"
    , plural $ comitative defaultInflection
    , [ (0, "nolline")
      , (1, "yksine")
      , (2, "kaksine")
      , (3, "kolmine")
      , (4, "neljine")
      , (5, "viisine")
      , (6, "kuusine")
      , (7, "seitsemine")
      , (8, "kahdeksine")
      , (9, "yhdeksine")
      , (10, "kymmenine")
      , (11, "yksinetoista")
      , (12, "kaksinetoista")
      , (13, "kolminetoista")
      , (14, "neljinetoista")
      , (15, "viisinetoista")
      , (16, "kuusinetoista")
      , (17, "seitseminetoista")
      , (18, "kahdeksinetoista")
      , (19, "yhdeksinetoista")
      , (20, "kaksinekymmenine")
      , (21, "kaksinekymmenineyksine")
      , (100, "satoine")
      , (1000, "tuhansine")
      ]
    )
  , ( "plural instructive"
    , plural $ instructive defaultInflection
    , [ (0, "nollin")
      , (1, "yksin")
      , (2, "kaksin")
      , (3, "kolmin")
      , (4, "neljin")
      , (5, "viisin")
      , (6, "kuusin")
      , (7, "seitsemin")
      , (8, "kahdeksin")
      , (9, "yhdeksin")
      , (10, "kymmenin")
      , (11, "yksintoista")
      , (12, "kaksintoista")
      , (13, "kolmintoista")
      , (14, "neljintoista")
      , (15, "viisintoista")
      , (16, "kuusintoista")
      , (17, "seitsemintoista")
      , (18, "kahdeksintoista")
      , (19, "yhdeksintoista")
      , (20, "kaksinkymmenin")
      , (21, "kaksinkymmeninyksin")
      , (100, "sadoin")
      , (1000, "tuhansin")
      ]
    )
  ]

ordinals :: (Integral i) => TestData i
ordinals =
  [ ( "singular nominative"
    , singular $ nominative defaultInflection
    , [ (0, "nollas")
      , (1, "ensimmäinen")
      , (2, "toinen")
      , (3, "kolmas")
      , (4, "neljäs")
      , (5, "viides")
      , (6, "kuudes")
      , (7, "seitsemäs")
      , (8, "kahdeksas")
      , (9, "yhdeksäs")
      , (10, "kymmenes")
      , (11, "yhdestoista")
      , (12, "kahdestoista")
      , (13, "kolmastoista")
      , (14, "neljästoista")
      , (15, "viidestoista")
      , (16, "kuudestoista")
      , (17, "seitsemästoista")
      , (18, "kahdeksastoista")
      , (19, "yhdeksästoista")
      , (20, "kahdeskymmenes")
      , (21, "kahdeskymmenesensimmäinen") -- also kahdeskymmenesyhdes
      , (22, "kahdeskymmenestoinen") -- also kahdeskymmeneskahdes
      , (23, "kahdeskymmeneskolmas")
      , (24, "kahdeskymmenesneljäs")
      , (25, "kahdeskymmenesviides")
      , (26, "kahdeskymmeneskuudes")
      , (27, "kahdeskymmenesseitsemäs")
      , (28, "kahdeskymmeneskahdeksas")
      , (29, "kahdeskymmenesyhdeksäs")
      , (30, "kolmaskymmenes")
      , (40, "neljäskymmenes")
      , (50, "viideskymmenes")
      , (60, "kuudeskymmenes")
      , (70, "seitsemäskymmenes")
      , (80, "kahdeksaskymmenes")
      , (90, "yhdeksäskymmenes")
      , (100, "sadas")
      , (101, "sadasensimmäinen")
      , (357, "kolmassadasviideskymmenesseitsemäs")
      , (1000, "tuhannes")
      , (1001, "tuhannesensimmäinen")
      , (3721, "kolmastuhannesseitsemässadaskahdeskymmenesensimmäinen")
      ]
    )
  , ( "singular accusative"
    , singular $ accusative defaultInflection
    , [ (0, "nollas")
      , (1, "ensimmäinen")
      , (2, "toinen")
      , (3, "kolmas")
      , (4, "neljäs")
      , (5, "viides")
      , (6, "kuudes")
      , (7, "seitsemäs")
      , (8, "kahdeksas")
      , (9, "yhdeksäs")
      , (10, "kymmenes")
      , (11, "yhdestoista")
      , (12, "kahdestoista")
      , (13, "kolmastoista")
      , (14, "neljästoista")
      , (15, "viidestoista")
      , (16, "kuudestoista")
      , (17, "seitsemästoista")
      , (18, "kahdeksastoista")
      , (19, "yhdeksästoista")
      , (20, "kahdeskymmenes")
      , (21, "kahdeskymmenesensimmäinen")
      , (100, "sadas")
      , (1000, "tuhannes")
      ]
    )
  , ( "singular genitive"
    , singular $ genitive defaultInflection
    , [ (0, "nollannen")
      , (1, "ensimmäisen")
      , (2, "toisen")
      , (3, "kolmannen")
      , (4, "neljännen")
      , (5, "viidennen")
      , (6, "kuudennen")
      , (7, "seitsemännen")
      , (8, "kahdeksannen")
      , (9, "yhdeksännen")
      , (10, "kymmenennen")
      , (11, "yhdennentoista")
      , (12, "kahdennentoista")
      , (13, "kolmannentoista")
      , (14, "neljännentoista")
      , (15, "viidennentoista")
      , (16, "kuudennentoista")
      , (17, "seitsemännentoista")
      , (18, "kahdeksannentoista")
      , (19, "yhdeksännentoista")
      , (20, "kahdennenkymmenennen")
      , (21, "kahdennenkymmenennenensimmäisen")
      , (100, "sadannen")
      , (357, "kolmannensadannenviidennenkymmenennenseitsemännen")
      , (1000, "tuhannennen")
      ]
    )
  , ( "singular partitive"
    , singular $ partitive defaultInflection
    , [ (0, "nollatta")
      , (1, "ensimmäistä")
      , (2, "toista")
      , (3, "kolmatta")
      , (4, "neljättä")
      , (5, "viidettä")
      , (6, "kuudetta")
      , (7, "seitsemättä")
      , (8, "kahdeksatta")
      , (9, "yhdeksättä")
      , (10, "kymmenettä")
      , (11, "yhdettätoista")
      , (12, "kahdettatoista")
      , (13, "kolmattatoista")
      , (14, "neljättätoista")
      , (15, "viidettätoista")
      , (16, "kuudettatoista")
      , (17, "seitsemättätoista")
      , (18, "kahdeksattatoista")
      , (19, "yhdeksättätoista")
      , (20, "kahdettakymmenettä")
      , (21, "kahdettakymmenettäensimmäistä")
      , (100, "sadatta")
      , (357, "kolmattasadattaviidettäkymmenettäseitsemätta")
      , (1000, "tuhannetta")
      ]
    )
  , ( "singular essive"
    , singular $ essive defaultInflection
    , [ (0, "nollantena")
      , (1, "ensimmmäisenä")
      , (2, "toisena")
      , (3, "kolmantena")
      , (4, "neljäntenä")
      , (5, "viidentenä")
      , (6, "kuudentena")
      , (7, "seitsemäntenä")
      , (8, "kahdeksantena")
      , (9, "yhdeksäntenä")
      , (10, "kymmenentenä")
      , (11, "yhdennessätoista")
      , (12, "kahdentenatoista")
      , (13, "kolmantenatoista")
      , (14, "neljäntenätoista")
      , (15, "viidentenätoista")
      , (16, "kuudentenatoista")
      , (17, "seitsemäntenätoista")
      , (18, "kahdeksantenatoista")
      , (19, "yhdeksäntenätoista")
      , (20, "kahdentenakymmenentenä")
      , (21, "kahdentenakymmenentenäensimmäisenä")
      , (100, "sadantena")
      , (357, "kolmantenasadantenaviidentenäkymmenentenäseitsemäntenä")
      , (1000, "tuhannentena")
      ]
    )
  , ( "singular translative"
    , singular $ translative defaultInflection
    , [ (0, "nollanneksi")
      , (1, "ensimmäiseksi")
      , (2, "toiseksi")
      , (3, "kolmanneksi")
      , (4, "neljänneksi")
      , (5, "viidenneksi")
      , (6, "kuudenneksi")
      , (7, "seitsemänneksi")
      , (8, "kahdeksanneksi")
      , (9, "yhdeksänneksi")
      , (10, "kymmenenneksi")
      , (11, "yhdenneksitoista")
      , (12, "kahdenneksitoista")
      , (13, "kolmanneksitoista")
      , (14, "neljänneksitoista")
      , (15, "viidenneksitoista")
      , (16, "kuudenneksitoista")
      , (17, "seitsemänneksitoista")
      , (18, "kahdenksanneksitoista")
      , (19, "yhdeksänneksitoista")
      , (20, "kahdenneksikymmenenneksi")
      , (21, "kahdenneksikymmenenneksiensimmäiseksi")
      , (100, "sadanneksi")
      , (1000, "tuhannenneksi")
      ]
    )
  , ( "singular locative inessive"
    , singular $ locativeInessive defaultInflection
    , [ (0, "nollannessa")
      , (1, "ensimmäisessä")
      , (2, "toisessa")
      , (3, "kolmannessa")
      , (4, "neljännessä")
      , (5, "viidennessä")
      , (6, "kuudennessa")
      , (7, "seitsemännessä")
      , (8, "kahdeksannessa")
      , (9, "yhdeksännessä")
      , (10, "kymmenennessä")
      , (11, "yhdennessätoista")
      , (12, "kahdennessatoista")
      , (13, "kolmannessatoista")
      , (14, "neljännessätoista")
      , (15, "viidennessätoista")
      , (16, "kuudennessatoista")
      , (17, "seitsemännessätoista")
      , (18, "kahdeksannessatoista")
      , (19, "yhdeksännessätoista")
      , (20, "kahdennessakymmenennessä")
      , (21, "kahdennessakymmenennessäensimmäisessä")
      , (100, "sadannessa")
      , (1000, "tuhannennessa")
      ]
    )
  , ( "singular locative elative"
    , singular $ locativeElative defaultInflection
    , [ (0, "nollannesta")
      , (1, "ensimmäisestä")
      , (2, "toisesta")
      , (3, "kolmannesta")
      , (4, "neljännestä")
      , (5, "viidennestä")
      , (6, "kuudennesta")
      , (7, "seitsemännestä")
      , (8, "kahdeksannesta")
      , (9, "yhdeksännestä")
      , (10, "kymmenennestä")
      , (11, "yhdennestätoista")
      , (12, "kahdennestatoista")
      , (13, "kolmannestatoista")
      , (14, "neljännestätoista")
      , (15, "viidennestätoista")
      , (16, "kuudennestatoista")
      , (17, "seitsemännestätoista")
      , (18, "kahdeksannestatoista")
      , (19, "yhdeksännestätoista")
      , (20, "kahdennestakymmenennestä")
      , (21, "kahdennestakymmenennestäensimmäisestä")
      , (100, "sadannesta")
      , (1000, "tuhannennesta")
      ]
    )
  , ( "singular locative illative"
    , singular $ locativeIllative defaultInflection
    , [ (0, "nollanteen")
      , (1, "ensimmäiseen")
      , (2, "toiseen")
      , (3, "kolmanteen")
      , (4, "neljänteen")
      , (5, "viidenteen")
      , (6, "kuudenteen")
      , (7, "seitsemänteen")
      , (8, "kahdeksanteen")
      , (9, "yhdeksänteen")
      , (10, "kymmenenteen")
      , (11, "yhdenteentoista")
      , (12, "kahdenteentoista")
      , (13, "kolmanteentoista")
      , (14, "neljänteentoista")
      , (15, "viidenteentoista")
      , (16, "kuudenteentoista")
      , (17, "seitsemänteentoista")
      , (18, "kahdeksanteentoista")
      , (19, "yhdeksänteentoista")
      , (20, "kahdenteenkymmenenteen")
      , (21, "kahdenteenkymmenenteenensimmäiseen")
      , (100, "sadanteen")
      , (1000, "tuhannenteen")
      ]
    )
  , ( "singular locative adessive"
    , singular $ locativeAdessive defaultInflection
    , [ (0, "nollannella")
      , (1, "ensimmäisellä")
      , (2, "toisella")
      , (3, "kolmannella")
      , (4, "neljännellä")
      , (5, "viidennellä")
      , (6, "kuudennella")
      , (7, "seitsemännellä")
      , (8, "kahdeksannella")
      , (9, "yhdeksännellä")
      , (10, "kymmenennellä")
      , (11, "yhdennellätoista")
      , (12, "kahdennellatoista")
      , (13, "kolmannellatoista")
      , (14, "neljännellätoista")
      , (15, "viidennellätoista")
      , (16, "kuudennellatoista")
      , (17, "seitsemännellätoista")
      , (18, "kahdeksannellatoista")
      , (19, "yhdeksännellätoista")
      , (20, "kahdennellakymmenennellä")
      , (21, "kahdennellakymmenennelläensimmäisellä")
      , (100, "sadannella")
      , (1000, "tuhannennella")
      ]
    )
  , ( "singular locative ablative"
    , singular $ locativeAblative defaultInflection
    , [ (0, "nollannelta")
      , (1, "ensimmäiseltä")
      , (2, "toiselta")
      , (3, "kolmannelta")
      , (4, "neljänneltä")
      , (5, "viidenneltä")
      , (6, "kuudennelta")
      , (7, "seitsemänneltä")
      , (8, "kahdeksannelta")
      , (9, "yhdeksänneltä")
      , (10, "kymmenenneltä")
      , (11, "yhdenneltätoista")
      , (12, "kahdenneltatoista")
      , (13, "kolmanneltatoista")
      , (14, "neljänneltätoista")
      , (15, "viidenneltätoista")
      , (16, "kuudenneltatoista")
      , (17, "seitsemänneltätoista")
      , (18, "kahdeksanneltatoista")
      , (19, "yhdeksänneltätoista")
      , (20, "kahdenneltakymmenenneltä")
      , (21, "kahdenneltakymmenenneltäensimmäiseltä")
      , (100, "sadannelta")
      , (1000, "tuhannennelta")
      ]
    )
  , ( "singular locative allative"
    , singular $ locativeAllative defaultInflection
    , [ (0, "nollannelle")
      , (1, "ensimmäiselle")
      , (2, "toiselle")
      , (3, "kolmannelle")
      , (4, "neljännelle")
      , (5, "viidennelle")
      , (6, "kuudennelle")
      , (7, "seitsemännelle")
      , (8, "kahdeksannelle")
      , (9, "yhdeksännelle")
      , (10, "kymmenennelle")
      , (11, "yhdennelletoista")
      , (12, "kahdennelletoista")
      , (13, "kolmannelletoista")
      , (14, "neljännelletoista")
      , (15, "viidennelletoista")
      , (16, "kuudennelletoista")
      , (17, "seitsemännelletoista")
      , (18, "kahdeksannelletoista")
      , (19, "yhdeksännelletoista")
      , (20, "kahdennellekymmenennelle")
      , (21, "kahdennellekymmenennelleensimmäiselle")
      , (100, "sadannelle")
      , (1000, "tuhannennelle")
      ]
    )
  , ( "singular abessive"
    , singular $ abessive defaultInflection
    , [ (0, "nollannetta")
      , (1, "ensimmäisettä")
      , (2, "toisetta")
      , (3, "kolmannetta")
      , (4, "neljännettä")
      , (5, "viidennettä")
      , (6, "kuudennetta")
      , (7, "seitsemännettä")
      , (8, "kahdeksannetta")
      , (9, "yhdeksännettä")
      , (10, "kymmenennettä")
      , (11, "yhdennettätoista")
      , (12, "kahdennettatoista")
      , (13, "kolmannettatoista")
      , (14, "neljännettätoista")
      , (15, "viidennettätoista")
      , (16, "kuudennettatoista")
      , (17, "seitsemännettätoista")
      , (18, "kahdeksannettatoista")
      , (19, "yhdeksännettätoista")
      , (20, "kahdennettakymmenennettä")
      , (21, "kahdennettakymmenennettäensimmäisettä")
      , (100, "sadannetta")
      , (1000, "tuhannennetta")
      ]
    )
  , ( "singular comitative"
    , singular $ comitative defaultInflection
    , [ (4, "neljänsine") ]
    )
  , ( "singular instructive"
    , singular $ instructive defaultInflection
    , [ (4, "neljänsin") ]
    )
  , ( "plural nominative"
    , plural $ nominative defaultInflection
    , [ (0, "nollannet")
      , (1, "ensimmäiset")
      , (2, "toiset")
      , (3, "kolmannet")
      , (4, "neljännet")
      , (5, "viidennet")
      , (6, "kuudennet")
      , (7, "seitsemännet")
      , (8, "kahdeksannet")
      , (9, "yhdeksännet")
      , (10, "kymmenennet")
      , (11, "yhdennettoista")
      , (12, "kahdennettoista")
      , (13, "kolmannettoista")
      , (14, "neljännettoista")
      , (15, "viidennettoista")
      , (16, "kuudennettoista")
      , (17, "seitsemännettoista")
      , (18, "kahdeksannettoista")
      , (19, "yhdeksännettoista")
      , (20, "kahdennetkymmenennet")
      , (21, "kahdennetkymmenennetensimmäiset")
      , (100, "sadannet")
      , (1000, "tuhannennet")
      ]
    )
  , ( "plural accusative"
    , plural $ accusative defaultInflection
    , [ (0, "nollannet")
      , (1, "ensimmäiset")
      , (2, "toiset")
      , (3, "kolmannet")
      , (4, "neljännet")
      , (5, "viidennet")
      , (6, "kuudennet")
      , (7, "seitsemännet")
      , (8, "kahdeksannet")
      , (9, "yhdeksännet")
      , (10, "kymmenennet")
      , (11, "yhdennettoista")
      , (12, "kahdennettoista")
      , (13, "kolmannettoista")
      , (14, "neljännettoista")
      , (15, "viidennettoista")
      , (16, "kuudennettoista")
      , (17, "seitsemännettoista")
      , (18, "kahdeksannettoista")
      , (19, "yhdeksännettoista")
      , (20, "kahdennetkymmenennet")
      , (21, "kahdennetkymmenennetensimmäiset")
      , (100, "sadannet")
      , (1000, "tuhannennet")
      ]
    )
  , ( "plural genitive"
    , plural $ genitive defaultInflection
    , [ (0, "nollansien")
      , (1, "ensimmäisten")
      , (2, "toisten")
      , (3, "kolmansien")
      , (4, "neljänsien")
      , (5, "viidensien")
      , (6, "kuudensien")
      , (7, "seitsemänsien")
      , (8, "kahdeksansien")
      , (9, "yhdeksänsien")
      , (10, "kymmenensien")
      , (11, "yhdensientoista")
      , (12, "kahdensientoista")
      , (13, "kolmansientoista")
      , (14, "neljänsientoista")
      , (15, "viidensientoista")
      , (16, "kuudensientoista")
      , (17, "seitsemänsientoista")
      , (18, "kahdeksansientoista")
      , (19, "yhdeksänsientoista")
      , (20, "kahdensienkymmenensien")
      , (21, "kahdensienkymmenensienensimmäisten")
      , (100, "sadansien")
      , (1000, "tuhannensien")
      ]
    )
  , ( "plural partitive"
    , plural $ partitive defaultInflection
    , [ (0, "nollansia")
      , (1, "ensimmäisiä")
      , (2, "toisia")
      , (3, "kolmansia")
      , (4, "neljänsiä")
      , (5, "viidensiä")
      , (6, "kuudensia")
      , (7, "seitsemänsiä")
      , (8, "kahdeksansia")
      , (9, "yhdeksänsiä")
      , (10, "kymmenensiä")
      , (11, "yhdensiätoista")
      , (12, "kahdensiatoista")
      , (13, "kolmansiatoista")
      , (14, "neljänsiätoista")
      , (15, "viidensiätoista")
      , (16, "kuudensiatoista")
      , (17, "seitsemänsiätoista")
      , (18, "kahdeksansiatoista")
      , (19, "yhdeksänsiätoista")
      , (20, "kahdensiakymmenensiä")
      , (21, "kahdensiakymmenensiäensimmäisiä")
      , (100, "sadansia")
      , (1000, "tuhannensia")
      ]
    )
  , ( "plural essive"
    , plural $ essive defaultInflection
    , [ (0, "nollansina")
      , (1, "ensimmäisinä")
      , (2, "toisina")
      , (3, "kolmansina")
      , (4, "neljänsinä")
      , (5, "viidensinä")
      , (6, "kuudensina")
      , (7, "seitsemänsinä")
      , (8, "kahdeksansina")
      , (9, "yhdeksänsinä")
      , (10, "kymmenensinä")
      , (11, "yhdensinätoista")
      , (12, "kahdensinatoista")
      , (13, "kolmansinatoista")
      , (14, "neljänsinätoista")
      , (15, "viidensinätoista")
      , (16, "kuudensinatoista")
      , (17, "seitsemänsinätoista")
      , (18, "kahdeksansinatoista")
      , (19, "yhdeksänsinätoista")
      , (20, "kahdensinakymmenensinä")
      , (21, "kahdensinakymmenensinäensimmäisinä")
      , (100, "sadansina")
      , (1000, "tuhannensina")
      ]
    )
  , ( "plural translative"
    , plural $ translative defaultInflection
    , [ (0, "nollansiksi")
      , (1, "ensimmäisiksi")
      , (2, "toisiksi")
      , (3, "kolmansiksi")
      , (4, "neljänsiksi")
      , (5, "viidensiksi")
      , (6, "kuudensiksi")
      , (7, "seitsemänsiksi")
      , (8, "kahdeksansiksi")
      , (9, "yhdeksänsiksi")
      , (10, "kymmenensiksi")
      , (11, "yhdensiksitoista")
      , (12, "kahdensiksitoista")
      , (13, "kolmansiksitoista")
      , (14, "neljänsiksitoista")
      , (15, "viidensiksitoista")
      , (16, "kuudensiksitoista")
      , (17, "seitsemänsiksitoista")
      , (18, "kahdeksansiksitoista")
      , (19, "yhdeksänsissätoista")
      , (20, "kahdensiksikymmenensiksi")
      , (21, "kahdensiksikymmenensiksiensimmäisiksi")
      , (100, "sadansiksi")
      , (1000, "tuhannensiksi")
      ]
    )
  , ( "plural locative inessive"
    , plural $ locativeInessive defaultInflection
    , [ (0, "nollansissa")
      , (1, "ensimmäisissä")
      , (2, "toisissa")
      , (3, "kolmansissa")
      , (4, "neljänsissä")
      , (5, "viidensissä")
      , (6, "kuudensissa")
      , (7, "seitsemänsissä")
      , (8, "kahdeksansissa")
      , (9, "yhdeksänsissä")
      , (10, "kymmenensissä")
      , (11, "yhdensissätoista")
      , (12, "kahdensissatoista")
      , (13, "kolmansissatoista")
      , (14, "neljänsissätoista")
      , (15, "viidensissätoista")
      , (16, "kuudensissatoista")
      , (17, "seitsemänsissätoista")
      , (18, "kahdeksansissatoista")
      , (19, "yhdeksänsissätoista")
      , (20, "kahdensissakymmenensissä")
      , (21, "kahdensissakymmenensissäensimmäisissä")
      , (100, "sadansissa")
      , (1000, "tuhannensissa")
      ]
    )
  , ( "plural locative elative"
    , plural $ locativeElative defaultInflection
    , [ (0, "nollansista")
      , (1, "ensimmäisistä")
      , (2, "toisista")
      , (3, "kolmansista")
      , (4, "neljänsistä")
      , (5, "viidensistä")
      , (6, "kuudensista")
      , (7, "seitsemänsistä")
      , (8, "kahdeksansista")
      , (9, "yhdeksänsistä")
      , (10, "kymmenensistä")
      , (11, "yhdensistätoista")
      , (12, "kahdensistatoista")
      , (13, "kolmansistatoista")
      , (14, "neljänsistätoista")
      , (15, "viidensistätoista")
      , (16, "kuudensistatoista")
      , (17, "seitsemänsistätoista")
      , (18, "kahdeksansistatoista")
      , (19, "yhdeksänsistätoista")
      , (20, "kahdensistakymmenensistä")
      , (21, "kahdensistakymmenensistäensimmäisistä")
      , (100, "sadansista")
      , (1000, "tuhannensista")
      ]
    )
  , ( "plural locative illative"
    , plural $ locativeIllative defaultInflection
    , [ (0, "nollansiin")
      , (1, "ensimmäisiin")
      , (2, "toisiin")
      , (3, "kolmansiin")
      , (4, "neljänsiin")
      , (5, "viidensiin")
      , (6, "kuudensiin")
      , (7, "seitsemänsiin")
      , (8, "kahdeksansiin")
      , (9, "yhdeksänsiin")
      , (10, "kymmenensiin")
      , (11, "yhdensiintoista")
      , (12, "kahdensiintoista")
      , (13, "kolmansiintoista")
      , (14, "neljänsiintoista")
      , (15, "viidensiintoista")
      , (16, "kuudensiintoista")
      , (17, "seitsemänsiintoista")
      , (18, "kahdeksansiintoista")
      , (19, "yhdeksänsiintoista")
      , (20, "kahdensiinkymmenensiin")
      , (21, "kahdensiinkymmenensiinensimmäisiin")
      , (100, "sadansiin")
      , (1000, "tuhannensiin")
      ]
    )
  , ( "plural locative adessive"
    , plural $ locativeAdessive defaultInflection
    , [ (0, "nollansilla")
      , (1, "ensimmäisillä")
      , (2, "toisilla")
      , (3, "kolmansilla")
      , (4, "neljänsillä")
      , (5, "viidensillä")
      , (6, "kuudensilla")
      , (7, "seitsemänsillä")
      , (8, "kahdeksansilla")
      , (9, "yhdeksänsillä")
      , (10, "kymmenensillä")
      , (11, "yhdensillätoista")
      , (12, "kahdensillatoista")
      , (13, "kolmansillatoista")
      , (14, "neljänsillätoista")
      , (15, "viidensillätoista")
      , (16, "kuudensillatoista")
      , (17, "seitsemänsillätoista")
      , (18, "kahdeksansillatoista")
      , (19, "yhdeksänsillätoista")
      , (20, "kahdensillakymmenensillä")
      , (21, "kahdensillakymmenensilläensimmäisillä")
      , (100, "sadansilla")
      , (1000, "tuhannensilla")
      ]
    )
  , ( "plural locative ablative"
    , plural $ locativeAblative defaultInflection
    , [ (0, "nollansilta")
      , (1, "ensimmäisiltä")
      , (2, "toisilta")
      , (3, "kolmansilta")
      , (4, "neljänsiltä")
      , (5, "viidensiltä")
      , (6, "kuudensilta")
      , (7, "seitsemänsiltä")
      , (8, "kahdeksansilta")
      , (9, "yhdeksänsiltä")
      , (10, "kymmenensiltä")
      , (11, "yhdensitlätoista")
      , (12, "kahdensiltatoista")
      , (13, "kolmansiltatoista")
      , (14, "neljänsiltätoista")
      , (15, "viidensiltätoista")
      , (16, "kuudensiltatoista")
      , (17, "seitsemänsiltätoista")
      , (18, "kahdeksansiltatoista")
      , (19, "yhdeksänsiltätoista")
      , (20, "kahdensiltakymmenensiltä")
      , (21, "kahdensiltakymmenensiltäensimmäisiltä")
      , (100, "sadansilta")
      , (1000, "tuhannensilta")
      ]
    )
  , ( "plural locative allative"
    , plural $ locativeAllative defaultInflection
    , [ (0, "nollansille")
      , (1, "ensimmäisille")
      , (2, "toisille")
      , (3, "kolmansille")
      , (4, "neljänsille")
      , (5, "viidensille")
      , (6, "kuudensille")
      , (7, "seitsemänsille")
      , (8, "kahdeksansille")
      , (9, "yhdeksänsille")
      , (10, "kymmenensille")
      , (11, "yhdensiintoista")
      , (12, "kahdensilletoista")
      , (13, "kolmansilletoista")
      , (14, "neljänsilletoista")
      , (15, "viidensilletoista")
      , (16, "kuudensilletoista")
      , (17, "seitsemänsilletoista")
      , (18, "kahdeksansilletoista")
      , (19, "yhdeksänsilletoista")
      , (20, "kahdensillekymmenensille")
      , (21, "kahdensillekymmenensilleensimmäisille")
      , (100, "sadansille")
      , (1000, "tuhannensille")
      ]
    )
  , ( "plural abessive"
    , plural $ abessive defaultInflection
    , [ (0, "nollansitta")
      , (1, "ensimmäisittä")
      , (2, "toisitta")
      , (3, "kolmansitta")
      , (4, "neljänsittä")
      , (5, "viidensittä")
      , (6, "kuudensitta")
      , (7, "seitsemänsittä")
      , (8, "kahdeksansitta")
      , (9, "yhdeksänsittä")
      , (10, "kymmenensittä")
      , (11, "yhdensittätoista")
      , (12, "kahdensittatoista")
      , (13, "kolmansittatoista")
      , (14, "neljänsittätoista")
      , (15, "viidensittätoista")
      , (16, "kuudensittatoista")
      , (17, "seitsemänsittätoista")
      , (18, "kahdeksansittatoista")
      , (19, "yhdeksänsittätoista")
      , (20, "kahdensittakymmenensittä")
      , (21, "kahdensittakymmenensittäensimmäisittä")
      , (100, "sadansitta")
      , (1000, "tuhannensitta")
      ]
    )
  , ( "plural comitative"
    , plural $ comitative defaultInflection
    , [ (0, "nollansine")
      , (1, "ensimmäisine")
      , (2, "toisine")
      , (3, "kolmansine")
      , (4, "neljänsine")
      , (5, "viidensine")
      , (6, "kuudensine")
      , (7, "seitsemänsine")
      , (8, "kahdeksansine")
      , (9, "yhdeksänsine")
      , (10, "kymmenensine")
      , (11, "yhdensinetoista")
      , (12, "kahdensinetoista")
      , (13, "kolmansinetoista")
      , (14, "neljänsinetoista")
      , (15, "viidensinetoista")
      , (16, "kuudensinetoista")
      , (17, "seitsemänsinetoista")
      , (18, "kahdeksansinetoista")
      , (19, "yhdeksänsinetoista")
      , (20, "kahdensinekymmenensine")
      , (21, "kahdensinekymmenensineensimmäisine")
      , (100, "sadansine")
      , (1000, "tuhannensine")
      ]
    )
  , ( "plural instructive"
    , plural $ instructive defaultInflection
    , [ (0, "nollansin")
      , (1, "ensimmäisin")
      , (2, "toisin")
      , (3, "kolmansin")
      , (4, "neljänsin")
      , (5, "viidensin")
      , (6, "kuudensin")
      , (7, "seitsemänsin")
      , (8, "kahdeksansin")
      , (9, "yhdeksänsin")
      , (10, "kymmenensin")
      , (11, "yhdensintoista")
      , (12, "kahdensintoista")
      , (13, "kolmansintoista")
      , (14, "neljänsintoista")
      , (15, "viidensintoista")
      , (16, "kuudensintoista")
      , (17, "seitsemänsintoista")
      , (18, "kahdeksansintoista")
      , (19, "yhdeksänsintoista")
      , (20, "kahdensinkymmenensin")
      , (21, "kahdensinkymmenensinensimmäisin")
      , (100, "sadansin")
      , (1000, "tuhannensin")
      ]
    )
  ]

multiplicatives :: (Integral i) => TestData i
multiplicatives =
  [ ( "default"
    , defaultInflection
    , [ (1, "kerran")
      , (2, "kahdesti")
      , (6, "kuudesti")
      ]
    )
  ]

multipliers :: (Integral i) => TestData i
multipliers =
  [ ( "default"
    , defaultInflection
    , [ (6, "kuusinkertainen")
      ]
    )
  ]
